package edu.unc.polygon.connector.amqp;

import org.identityconnectors.common.logging.Log;
import org.identityconnectors.common.security.GuardedString;
import org.identityconnectors.framework.common.exceptions.ConfigurationException;
import org.identityconnectors.framework.common.exceptions.ConnectorException;
import org.identityconnectors.framework.common.objects.*;
import org.identityconnectors.framework.common.objects.filter.FilterTranslator;
import org.identityconnectors.framework.spi.Configuration;
import org.identityconnectors.framework.spi.Connector;
import org.identityconnectors.framework.spi.ConnectorClass;
import org.identityconnectors.framework.spi.operations.*;
import org.identityconnectors.common.Base64;
import org.identityconnectors.common.StringUtil;
import org.identityconnectors.common.security.GuardedByteArray;
import org.identityconnectors.common.security.GuardedString;

import com.rabbitmq.client.*;

import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.io.*;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;


@ConnectorClass(displayNameKey = "UI_AMQP_CONNECTOR_NAME", configurationClass = AmqpConfiguration.class)
public class AmqpConnector implements  Connector, DeleteOp, SearchOp<AmqpFilter>, SchemaOp, TestOp {

public static final String ATTR_MAIL = "email";
private static final Log LOG = Log.getLog(AmqpConnector.class);

    private AmqpConfiguration configuration;

    @Override
    public Configuration getConfiguration() {
        return configuration;
    }


    @Override
    public void init(Configuration configuration) {
        LOG.info(">>> Initializing connector");

        if (!(configuration instanceof AmqpConfiguration)) {
            throw new ConfigurationException("Configuration is not instance of " + AmqpConfiguration.class.getName());
        }

        AmqpConfiguration amqpConfig = (AmqpConfiguration) configuration;
        amqpConfig.validate();
        this.configuration = amqpConfig;
        LOG.info(">>> Connector initialization finished");
    }
    @Override
    public void dispose() {
        configuration = null;
        //handler = null;
    }
    @Override
    public FilterTranslator<AmqpFilter> createFilterTranslator(ObjectClass oc, OperationOptions oo) {
        LOG.info("inside  createFilterTranslator");
        return new AmqpFilterTranslator();
    }
        private boolean isUniqueAndNameAttributeEqual() {
        LOG.info("inside  isUniqueAndNameAttributeEqual");
        String uniqueAttribute = configuration.getAmqpUniqueAttribute();
        String nameAttribute = configuration.getAmqpNameAttribute();

        return uniqueAttribute == null ? nameAttribute == null : uniqueAttribute.equals(nameAttribute);
    }
    @Override
    public void test() {
      try {
          ConnectionFactory factory = new ConnectionFactory();
          factory.setHost(configuration.getAmqpHost());
          factory.setUsername(configuration.getAmqpUsername());
          factory.setPassword(configuration.getAmqpPassword());
          Connection connection = factory.newConnection();
          Channel channel = connection.createChannel();
          boolean connected = channel.isOpen();
          if (connected) {
              LOG.info("Testing connection succeeded");
          }
          else {
              LOG.info("Testing connection failed");
          }
          connection.close();
      } catch (Exception teste) {
         LOG.info("in exception teste" + teste.getStackTrace());
         //connection.close();
      }
        
    }
    @Override
     public Schema schema() {
        LOG.info(">>> schema started");

        SchemaBuilder builder = new SchemaBuilder(AmqpConnector.class);
        Set<AttributeInfo> attributes= new HashSet<AttributeInfo>();
        AttributeInfoBuilder id = new AttributeInfoBuilder();
        id.setName("id");
        id.setCreateable(true);
        id.setUpdateable(true);
        id.setReadable(true);
        id.setRequired(false);
        id.setMultiValued(false);
        attributes.add(id.build());
        AttributeInfoBuilder sourceId = new AttributeInfoBuilder();
        sourceId.setName("sourceId");
        sourceId.setCreateable(true);
        sourceId.setUpdateable(true);
        sourceId.setReadable(true);
        sourceId.setRequired(false);
        sourceId.setMultiValued(true);
        attributes.add(sourceId.build());
        AttributeInfoBuilder priorSubjectId = new AttributeInfoBuilder();
        priorSubjectId.setName("priorSubjectId");
        priorSubjectId.setCreateable(true);
        priorSubjectId.setUpdateable(true);
        priorSubjectId.setReadable(true);
        priorSubjectId.setRequired(false);
        priorSubjectId.setMultiValued(true);
        attributes.add(priorSubjectId.build());
        AttributeInfoBuilder personalPronoun = new AttributeInfoBuilder();
        personalPronoun.setName("personalPronoun");
        personalPronoun.setCreateable(true);
        personalPronoun.setUpdateable(true);
        personalPronoun.setReadable(true);
        personalPronoun.setRequired(false);
        personalPronoun.setMultiValued(false);
        attributes.add(personalPronoun.build());
        AttributeInfoBuilder subjectId = new AttributeInfoBuilder();
        subjectId.setName("subjectId");
        subjectId.setCreateable(true);
        subjectId.setUpdateable(true);
        subjectId.setReadable(true);
        subjectId.setRequired(false);
        subjectId.setMultiValued(false);
        attributes.add(subjectId.build());
        AttributeInfoBuilder loginId = new AttributeInfoBuilder();
        loginId.setName("loginId");
        loginId.setCreateable(true);
        loginId.setUpdateable(true);
        loginId.setReadable(true);
        loginId.setRequired(false);
        loginId.setMultiValued(false);
        attributes.add(loginId.build());
        AttributeInfoBuilder birthDate = new AttributeInfoBuilder();
        birthDate.setName("birthDate");
        birthDate.setCreateable(true);
        birthDate.setUpdateable(true);
        birthDate.setReadable(true);
        birthDate.setRequired(false);
        birthDate.setMultiValued(false);
        attributes.add(birthDate.build());
        AttributeInfoBuilder nameType = new AttributeInfoBuilder();
        nameType.setName("nameType");
        nameType.setCreateable(true);
        nameType.setUpdateable(true);
        nameType.setReadable(true);
        nameType.setRequired(false);
        nameType.setMultiValued(false);
        attributes.add(nameType.build());
        AttributeInfoBuilder middleName = new AttributeInfoBuilder();
        middleName.setName("middleName");
        middleName.setCreateable(true);
        middleName.setUpdateable(true);
        middleName.setReadable(true);
        middleName.setRequired(false);
        middleName.setMultiValued(false);
        attributes.add(middleName.build());
        AttributeInfoBuilder givenName = new AttributeInfoBuilder();
        givenName.setName("givenName");
        givenName.setCreateable(true);
        givenName.setUpdateable(true);
        givenName.setReadable(true);
        givenName.setRequired(false);
        givenName.setMultiValued(false);
        attributes.add(givenName.build());
        AttributeInfoBuilder familyName = new AttributeInfoBuilder();
        familyName.setName("familyName");
        familyName.setCreateable(true);
        familyName.setUpdateable(true);
        familyName.setReadable(true);
        familyName.setRequired(false);
        familyName.setMultiValued(false);
        attributes.add(familyName.build());
        AttributeInfoBuilder fullname = new AttributeInfoBuilder();
        fullname.setName("fullname");
        fullname.setCreateable(true);
        fullname.setUpdateable(true);
        fullname.setReadable(true);
        fullname.setRequired(false);
        fullname.setMultiValued(false);
        attributes.add(fullname.build());
        AttributeInfoBuilder firstname = new AttributeInfoBuilder();
        firstname.setName("firstname");
        firstname.setCreateable(true);
        firstname.setUpdateable(true);
        firstname.setReadable(true);
        firstname.setRequired(false);
        firstname.setMultiValued(false);
        attributes.add(firstname.build());
        AttributeInfoBuilder lastname = new AttributeInfoBuilder();
        lastname.setName("lastname");
        lastname.setCreateable(true);
        lastname.setUpdateable(true);
        lastname.setReadable(true);
        lastname.setRequired(false);
        lastname.setMultiValued(false);
        attributes.add(lastname.build());
        AttributeInfoBuilder orgId = new AttributeInfoBuilder();
        orgId.setName("orgId");
        orgId.setCreateable(true);
        orgId.setUpdateable(true);
        orgId.setReadable(true);
        orgId.setRequired(false);
        orgId.setMultiValued(false);
        attributes.add(orgId.build());
        AttributeInfoBuilder job = new AttributeInfoBuilder();
        job.setName("job");
        job.setCreateable(true);
        job.setUpdateable(true);
        job.setReadable(true);
        job.setRequired(false);
        job.setMultiValued(false);
        attributes.add(job.build());
        AttributeInfoBuilder email = new AttributeInfoBuilder();
        email.setName("email");
        email.setCreateable(true);
        email.setUpdateable(true);
        email.setReadable(true);
        email.setRequired(false);
        email.setMultiValued(false);
        attributes.add(email.build());
        AttributeInfoBuilder phone = new AttributeInfoBuilder();
        phone.setName("phone");
        phone.setCreateable(true);
        phone.setUpdateable(true);
        phone.setReadable(true);
        phone.setRequired(false);
        phone.setMultiValued(false);
        attributes.add(phone.build());
        AttributeInfoBuilder active = new AttributeInfoBuilder();
        active.setName("active");
        active.setCreateable(true);
        active.setUpdateable(true);
        active.setReadable(true);
        active.setRequired(false);
        active.setMultiValued(false);
        attributes.add(active.build());
        builder.defineObjectClass(ObjectClass.ACCOUNT.getDisplayNameKey(),attributes);        
        LOG.info(">>> schema finished");
        return builder.build();
    }

    @Override
    public void executeQuery(ObjectClass oc, AmqpFilter filter, ResultsHandler handler, OperationOptions oo) {
        LOG.info(">>> executeQuery ");
        try {
            LOG.info("about to create factory");
            ConnectionFactory factory = new ConnectionFactory();
            factory.setHost(configuration.getAmqpHost());
            factory.setUsername(configuration.getAmqpUsername());
            factory.setPassword(configuration.getAmqpPassword());
            Connection connection = factory.newConnection();
            Channel channel = connection.createChannel();
            boolean durable = true;
            channel.exchangeDeclare(configuration.getAmqpExchangename(), BuiltinExchangeType.TOPIC, durable);
            String queueName = channel.queueDeclare().getQueue();
            queueName = configuration.getAmqpQueuename();
            LOG.info("about to bind queue" + queueName + configuration.getAmqpExchangename() + configuration.getAmqpRoutingkey());
            channel.queueBind(queueName,configuration.getAmqpExchangename(),configuration.getAmqpRoutingkey());
            boolean connected = channel.isOpen();
            if (connected) {
                LOG.info("Testing connection succeeded");
            }
            else {
                LOG.info("Testing connection failed");
            }
            boolean autoAck = false;
            while (true) {
            LOG.info("about to get response");
            GetResponse response = channel.basicGet(queueName, autoAck);
            /* A null would mean no messages are available to process */
               if(response != null) {
                  String message = "";
                  message = new String(response.getBody(), "UTF-8");
                  LOG.info("just got the message body");
                  long deliveryTag = response.getEnvelope().getDeliveryTag();
                  channel.basicAck(deliveryTag, false);
                  Reader in = new StringReader(message);
                  String id = "";
                  String familyName = "";
                  String givenName = "";
                  String middleName = "";
                  String birthDate = "";
                  String logonId = "";
                  String subjectId = "";
                  String priorSubjectId = "";
                  String sourceId = "";
                  String personalPronoun = "";
                  String orgId = "";
                  String job = "";
                  String email = "";
                  String phone = "";
                  String active = "true";
                  try {
                      JSONParser jsonParser = new JSONParser();
                      JSONObject jsonObject = (JSONObject) jsonParser.parse(message);
                      LOG.info("just did initial object parse of message");
                      JSONObject scimMessage = (JSONObject) jsonObject.get(configuration.getAmqpScimPayloadField());
                      LOG.info("just got the scim message");
                      LOG.info("payload field " + configuration.getAmqpScimPayloadField());
                      id = (String) scimMessage.get(configuration.getAmqpNameAttribute());
                      LOG.info("id " + id);
                      LOG.info("about to get names");
                      JSONArray names = (JSONArray)  scimMessage.get("names");
                      LOG.info("got names");
                      Iterator<?> i = names.iterator();
                      while (i.hasNext()) {
                          LOG.info("inside name iterator");
                          JSONObject name = (JSONObject) i.next();
                          fullname = (String) name.get("formatted");
                          LOG.info(fullname);
                          firstname = (String) name.get("givenName");
                          LOG.info(firstname);
                          lastname = (String) name.get("familyName");
                          LOG.info(lastname);
                          break;
                      }
                      LOG.info("finished names");
                      JSONArray emails = (JSONArray)  scimMessage.get("emails");
                      LOG.info("got emails");
                      i = emails.iterator();
                      while (i.hasNext()) {
                      JSONObject emailEntry = (JSONObject) i.next();
                          email = (String) emailEntry.get("value");
                          String type = (String) emailEntry.get("type");
                          if (type.equals("primary")) {
                              break;
                          }
                      }
                      JSONArray phoneNumbers = (JSONArray)  scimMessage.get("phoneNumbers");
                      i = phoneNumbers.iterator();
                      while (i.hasNext()) {
                          JSONObject phoneNumber = (JSONObject) i.next();
                          phone = (String) phoneNumber.get("value");
                          break;
                      }
                  LOG.info("done parsing json");
                  } catch (Exception e) {
                      LOG.info("Exception parsing json" + e.getMessage());
                      LOG.info("Exception parsing json" + e.getCause());
                      LOG.info("Exception parsing json" + e.getStackTrace());
                  }

                  HashMap<String, String> record = new HashMap<String, String>();
                  record.put("id",id);
                  record.put("fullname",fullname);
                  record.put("lastname",lastname);
                  record.put("firstname",firstname);
                  record.put("orgId",orgId);
                  record.put("job",job);
                  record.put("email",email);
                  record.put("phone",phone);
                  record.put("active",active);
                   
                  LOG.info("about to call createConnectorObject");
                  ConnectorObject obj = createConnectorObject(record);
                  LOG.info("about to call handler");
                  //ResultsHandler handler = new ResultsHandler();
                  handler.handle(obj);
                  LOG.info("finished call handler");

               } else {
                  LOG.info("message response was null");
                  break;
               }
            }
            connection.close();
         } catch (Exception queryEx) {
            LOG.info("in exception queryEx" + queryEx.getMessage());
            LOG.info("in exception queryEx" + queryEx.getCause());
            LOG.info("in exception queryEx" + queryEx.getStackTrace());
            //connection.close();
         }
        LOG.info(">>> executeQuery finished");
    }
        private ConnectorObject createConnectorObject(HashMap<String, String> record) {
        ConnectorObjectBuilder builder = new ConnectorObjectBuilder();
        LOG.info("inside  createConnectorObject ");

        for ( Map.Entry<String, String> entry  :  record.entrySet()) {
            String name = entry.getKey();
            String value = record.get(name);
            LOG.info("inside record breakdown in createConnectorObject name == " +  name +  "value == " +  value);
            if (StringUtil.isEmpty(value)) {
                continue;
            }

            if (name.equals(configuration.getAmqpUniqueAttribute())) {
                builder.setUid(value);

                if (!isUniqueAndNameAttributeEqual()) {
                    continue;
                }
            }

            if (name.equals(configuration.getAmqpNameAttribute())) {
                builder.setName(new Name(value));
                continue;
            }

            if (name.equals(configuration.getAmqpPasswordAttribute())) {
                builder.addAttribute(OperationalAttributes.PASSWORD_NAME, new GuardedString(value.toCharArray()));
                continue;
            }

            builder.addAttribute(name, value);
        }

        LOG.info("build completed");
        return builder.build();
    }

    @Override
    public void delete(ObjectClass oc, Uid uid, OperationOptions oo) {
        LOG.info(">>> delete {0} {1} {2}", oc, uid, oo);

        //getHandler(oc).delete(oc, uid, oo);

        LOG.info(">>> delete finished");
    }

}
